<?php
/**
 * Example to use myMVC.
 * This program just demonstrate how to create
 * a login form, authenticate user, check data from
 * a form and read data from database.
 * 
 * Author: Luca Bertoncello <lucabert@lucabert.de>
 */

/**
 * Login-Actionclass. It <b>must</b> be derived from
 * myMVC_BaseLoginAction.
 * You can see how to define a validate-Function (setCheckFunction)
 * and how to validate data (validatePost).
 * In the perform-function, if the user logged in, you <b>must</b>
 * call $this->setLoggedIn($user);
 * $user contains an instance of a myMVC_BaseLogin derived class.
 * After a successfully login, you can call setNextAction() to
 * redirect to the main action or, if the user called it (eg. from bookmark)
 * to the wanted action.
 */
class LoginAction extends myMVC_BaseLoginAction
{
  public function __construct($request, $session)
  {
    parent::__construct($request, $session);
    $this->setCheckFunction('POST', 'validatePost');
  }

  public static function getActionGroup()
  {
    return 'login';
  }

  /**
   * Validate the data from user, coming as POST (form submit).
   * 
   * @return boolean                true if data are valid, false otherwise
   */
  public function validatePost()
  {
    $definitions = array();
    if($this->request->getSubmitParameter('loginButton') != '')
      $definitions = array(
          'login'   => array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'empty' => false),
          'passwd'  => array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'minlen' => 3, 'empty' => false),
      );
    elseif($this->request->getSubmitParameter('changeLanguage') != '')
      $definitions = array(
          'lang'    => array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'empty' => false, 'range' => array_keys(myMVC_Controller::getAvailableLanguages())),
      );
    $names = array(
        'login'     => _('Benutzername'),
        'passwd'    => _('Passwort'),
        'lang'      => _('Sprache'),
    );
    $errorMsgs = array(
        'login'     => array(myMVC_BaseAction::$ERROR_NOTSTRING => sprintf(_("%s muss eine E-Mail-Adresse sein"), '{field}', '{checkValue}'),
                             myMVC_BaseAction::$ERROR_EMPTY     => sprintf(_("%s kann nicht leer sein"), '{field}')),
        'passwd'    => array(myMVC_BaseAction::$ERROR_MINLEN    => sprintf(_("%s muss mindestens %s Zeichen lang sein"), '{field}', '{checkValue}'),
                             myMVC_BaseAction::$ERROR_EMPTY     => sprintf(_("%s kann nicht leer sein"), '{field}')),
        'lang'      => array(myMVC_BaseAction::$ERROR_NOTSTRING => sprintf(_("%s muss eine E-Mail-Adresse sein"), '{field}', '{checkValue}'),
                             myMVC_BaseAction::$ERROR_EMPTY     => sprintf(_("%s kann nicht leer sein"), '{field}'),
                             myMVC_BaseAction::$ERROR_RANGE     => sprintf(_("%s ist ung&uuml;ltig"), '{field}')),
    );
    return parent::validateInput($definitions, $names, $errorMsgs);
  }

  /**
   * Perform the Action
   * 
   * @return myMVC_HttpResponse     Response code for the next Action or View
   */
  public function perform()
  {
    $response = new myMVC_HttpResponse();
    if($this->request->getRequestMethod() == 'GET')
      $response->setNext('LoginView');
    else
    {
      if($this->request->getSubmitParameter('login') != '')
      {
        $user = Account::findByPK(array('accountname' => $this->request->getParameter('login')));
        if($user !== NULL)
        {
          if($user->getPasswd() == $this->request->getParameter('passwd'))
          {
            $this->setLoggedIn($user);
            $this->setNextAction($response);
          }
          else
          {
            $response->setNext('LoginView');
            $errs = $this->session->getFromNamespace('FORMCHECK', 'ERRORS');
            $errs['login'] = _('Ung&uuml;ltiger Benutzername oder Passwort');
            $this->session->setInNamespace('FORMCHECK', 'ERRORS', $errs);
          }
        }
        else
        {
          $response->setNext('LoginView');
          $errs = $this->session->getFromNamespace('FORMCHECK', 'ERRORS');
          $errs['login'] = _('Ung&uuml;ltiger Benutzername oder Passwort');
          $this->session->setInNamespace('FORMCHECK', 'ERRORS', $errs);
        }
      }
    }
    return $response;
  }

  /**
   * Called if user data are not valid (check with validate)
   * 
   * @return myMVC_HttpResponse     Response code for the next Action or View
   */
  public function manageInvalidData()
  {
    $response = new myMVC_HttpResponse();
    $response->setNext('LoginView');
    return $response;
  }
}
?>
