<?php
class ManageAliasAction extends GUIAction
{
  public function __construct($request, $session)
  {
    parent::__construct($request, $session);
    $this->setCheckFunction('POST', 'validatePost');
    $this->setCheckFunction('GET', 'validateGet');
    $this->setCheckFunction('AJAX', 'validateAjax');
  }

  /**
   * Returns the CSS class of this action
   * 
   * @return string                     The CSS class
   */
  public static function getCSSClass()
  {
    return 'general';
  }

  /**
   * Returns the token used to identify the current Action
   * 
   * @return string                 The token
   */
  public static function getActionToken()
  {
    return 'manageAlias';
  }

  /**
   * Returns whether this action should appear in the menu (created by controller)
   * 
   * @return boolean                true if the action should appear in menu
   */
  public static function isMenuAction()
  {
    return true;
  }

  /**
   * Returns the position of this action in the menu (created by controller)
   * Only necessary if isMenuAction() returns true
   * 
   * @return int                        The position in the menu (lower is "upper")
   */
  public static function getActionMenuPosition()
  {
    return 30;
  }

  /**
   * Returns the string to be displayed in the menu to identify the current action.
   * Only necessary if isMenuAction() returns true
   * 
   * @return string                 The menu item
   */
  public static function getActionMenuName()
  {
    return _("Aliase");
  }

  /**
   * Gibt den Name des Bildes für dieses Menüpunkt zurücl
   * 
   * @return string                 The menu item
   */
  public static function getActionMenuImage()
  {
    return '';
  }

  /**
   * Returns whether the current user is allowed to call the current Action
   * 
   * @return boolean                true if the user may call this Action, false otherwise
   */
  public static function hasUserRightForAction()
  {
    $user = self::getLoggedUser();
    if($user !== NULL && ($user->getIsAdmin() != 'f' || $user->getCanModifyAlias() == 't'))
      return true;
    return false;
  }

  /**
   * Validate the data from user, coming as GET
   *
   * @return boolean                true if data are valid, false otherwise
   */
  public function validateGet()
  {
    $this->session->useNamespace('manageAlias');
    $acc = $this->session->get('usedAccount');

    if($this->request->getSubmitParameter('delete') != '')
    {
      $user = self::getLoggedUser();
      $definitions = array(
          'delete' => array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'empty' => false,
                                        'comparedb' => array('query' => sprintf("SELECT COUNT(*) FROM aliases WHERE accountname = '%s' AND aliasname = {checkValue}",
                                                                          $acc->getAccountname()),
                                            'compareType' => '>=', 'compareValue' => 1)),
          'domain' => array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'empty' => false,
                                        'comparedb' => array('query' => 'SELECT COUNT(*) FROM domain WHERE domainname = {checkValue}',
                                            'compareType' => '==', 'compareValue' => 1)),
      );
    }
    $names = array(
          'delete' => _('Alias zu l&ouml;schen'),
          'domain' => _('Domain'),
    );
    $errorMsgs = array(
          'delete' => array(myMVC_BaseAction::$ERROR_COMPAREDB => _('Der Alias existiert nicht'),
                            myMVC_BaseAction::$ERROR_EMPTY     => sprintf(_("%s kann nicht leer sein"), '{field}'),
                            myMVC_BaseAction::$ERROR_NOTSTRING => sprintf(_("%s ist ung&uuml;ltig"), '{field}')),
          'domain' => array(myMVC_BaseAction::$ERROR_COMPAREDB => _('Die Domain existiert nicht'),
                            myMVC_BaseAction::$ERROR_EMPTY     => sprintf(_("%s kann nicht leer sein"), '{field}'),
                            myMVC_BaseAction::$ERROR_NOTSTRING => sprintf(_("%s ist ung&uuml;ltig"), '{field}')),
    );

    if(!is_array($definitions))
      return true;
    return parent::validateInput($definitions, $names, $errorMsgs);
  }

  /**
   * Validate the data from user, coming as POST
   *
   * @return boolean                true if data are valid, false otherwise
   */
  public function validatePost()
  {
    $session = myMVC_HttpSession::getInstance();
    $domains = array();
    foreach($session->getFromNamespace('manageAlias', 'domains') as $d)
      array_push($domains, $d->getDomainname());
    $user = self::getLoggedUser();
    $users = array('__SELF__');
    foreach($user->getUsersForAdmin() as $u)
      array_push($users, $u->getAccountname());
    if($this->request->getSubmitParameter('action') == 'chgAdminUser')
    {
      $definitions = array(
          'adminUser'           => array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'empty' => false, 'range' => $users),
      );
    }
    else
    {
      $definitions = array(
          'newValue'            => array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'empty' => false, 'maxlen' => 255, 'regexp' => '/^[A-z0-9][\w+.-]*$/'),
          'newDomain'           => array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'empty' => false, 'range' => $domains),
      );
      if($user->getIsAdmin() != 'f')
      {
        $users = array('__SELF__');
        foreach($user->getUsersForAdmin() as $u)
          array_push($users, $u->getAccountname());
        $definitions['adminUser'] =  array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'empty' => false, 'range' => $users);
      }
    }
    $names = array(
        'newValue'              => _('Alias'),
        'newDomain'             => _('Domain'),
        'adminUser'             => _('Administrierter Nutzer'),
    );
    $errorMsgs = array(
        'newValue'              => array(myMVC_BaseAction::$ERROR_MAXLEN    => sprintf(_("%s ist zu lang"), '{field}'),
                                         myMVC_BaseAction::$ERROR_EMPTY     => sprintf(_("%s kann nicht leer sein"), '{field}'),
                                         myMVC_BaseAction::$ERROR_NOTSTRING => sprintf(_("%s ist ung&uuml;ltig"), '{field}'),
                                         myMVC_BaseAction::$ERROR_REGEXP    => sprintf(_("%s ist ung&uuml;ltig"), '{field}')),
        'newDomain'             => array(myMVC_BaseAction::$ERROR_RANGE     => sprintf(_("%s ist ung&uuml;ltig"), '{field}'),
                                         myMVC_BaseAction::$ERROR_EMPTY     => sprintf(_("%s kann nicht leer sein"), '{field}'),
                                         myMVC_BaseAction::$ERROR_NOTSTRING => sprintf(_("%s ist ung&uuml;ltig"), '{field}')),
        'adminUser'             => array(myMVC_BaseAction::$ERROR_EMPTY         => sprintf(_("%s kann nicht leer sein"), '{field}'),
                                         myMVC_BaseAction::$ERROR_RANGE         => sprintf(_("%s ist ung&uuml;ltig"), '{field}'),
                                         myMVC_BaseAction::$ERROR_NOTSTRING     => sprintf(_("%s ist ung&uuml;ltig"), '{field}')),
    );
    return parent::validateInput($definitions, $names, $errorMsgs);
  }

  /**
   * Validate the data from user, coming as AJAX.
   *
   * @return boolean                true if data are valid, false otherwise
   */
  public function validateAjax()
  {
    $definitions = array(
        'search'     => array('type' => myMVC_BaseAction::$VALIDATE_STRING, 'minlen' => 2, 'maxlen' => 255, 'empty' => true),
    );
    $names = array(
        'search'     => _('Suchkriterium'),
    );
    $errorMsgs = array(
        'search'     => array(myMVC_BaseAction::$ERROR_NOTSTRING => sprintf(_("%s ist ung&uuml;ltig"), '{field}'),
                              myMVC_BaseAction::$ERROR_MINLEN    => sprintf(_("%s muss zwischen 3 und 255 Zeichen lang sein"), '{field}'),
                              myMVC_BaseAction::$ERROR_MAXLEN    => sprintf(_("%s muss zwischen 3 und 255 Zeichen lang sein"), '{field}')),
    );
    return parent::validateInput($definitions, $names, $errorMsgs);
  }

  /**
   * Perform the Action
   * 
   * @return myMVC_HttpResponse     Response code for the next Action or View
   */
  public function perform()
  {
    global $config;

    $response = new myMVC_HttpResponse();

    $response->setNext('ManageAliasView');
    $this->session->useNamespace('manageAlias');
    $user = self::getLoggedUser();

    if($this->request->getRequestMethod() == 'GET')
    {
      if($this->isCalledFromMenu())
      {
        $this->session->emptyNamespace();
        $acc = Account::findByPK(array('accountname' => $user->getAccountname()));
        $this->session->set('usedAccount', $acc);
      }
      else
        $acc = $this->session->get('usedAccount');
      if($user->getIsAdmin() == 't')
        $this->session->set('domains', Domain::callFinder('findAll', array()));
      else
        $this->session->set('domains', Domain::getDomainsForUser($acc->getAccountname()));

      if($this->request->getSubmitParameter('delete') != '')
      {
        $alias = Aliases::findByPK(array('accountname' => $acc->getAccountname(),
                                         'aliasname'   => $this->request->getParameter('delete'),
                                         'domainname'  => $this->request->getParameter('domain')));
        $alias->delete();
        $this->saveParameterForNextPage('popupMessage', _('Die Adresse wurde erfolgreich gel&ouml;scht'));
        if($user->getIsAdmin() != 'f')
          $this->saveParameterForNextPage('adminUser', $acc->getAccountname());
      }
    }
    else
    {
      if($this->request->getSubmitParameter('action') == 'chgAdminUser')
      {
        if($this->request->getParameter('adminUser') != '__SELF__')
          $acc = Account::findByPK(array('accountname' => $this->request->getParameter('adminUser')));
        else
          $acc = Account::findByPK(array('accountname' => $user->getAccountname()));
        $this->session->set('usedAccount', $acc);
        $this->saveParameterForNextPage('account', $acc);
        $this->saveParameterForNextPage('adminUser', $this->request->getParameter('adminUser'));
      }
      else
      {
        $acc = $this->session->get('usedAccount');
        $alias = new Aliases();
        $alias->setAccountname($acc->getAccountname());
        $alias->setDomainname($this->request->getParameter('newDomain'));
        $alias->setAliasname($this->request->getParameter('newValue'));
        $alias->save();
        $this->saveParameterForNextPage('popupMessage', _('Die Adresse wurde hinzugef&uuml;t und ist sofort nutzbar'));
        if($user->getIsAdmin() != 'f')
          $this->saveParameterForNextPage('adminUser', $this->request->getParameter('adminUser'));
        else
          $acc = Account::findByPK(array('accountname' => $user->getAccountname()));
        $this->saveParameterForNextPage('account', $acc);
      }
    }
    $this->saveParameterForNextPage('aliases', Aliases::callFinder('findByAccountname', array('accountname' => $acc->getAccountname()), 'domainname ASC, aliasname ASC'));

    return $response;
  }

  /**
   * Called if user data are not valid (check with validate)
   * 
   * @return myMVC_HttpResponse     Response code for the next Action or View
   */
  public function manageInvalidData()
  {
    $user = self::getLoggedUser();
    $response = new myMVC_HttpResponse();
    $response->setNext('ManageAliasView');
    $this->saveParameterForNextPage('error', true);
    $this->session->useNamespace('manageAlias');
    $acc = $this->session->get('usedAccount');
    $this->saveParameterForNextPage('aliases', Aliases::callFinder('findByAccountname', array('accountname' => $acc->getAccountname()), 'domainname ASC, aliasname ASC'));

    return $response;
  }

  /**
   * Perform the AJAX-Action
   *
   * @return myMVC_HttpResponse     Response code for the next Action or View
   */
  public function performAjax()
  {
    global $controller;

    $this->session->useNamespace('manageAlias');
    header('Content-Type: application/json');
    $user = self::getLoggedUser();
    $acc = $this->session->get('usedAccount');

    if($this->request->getParameter('search') != '')
      $aliases = Aliases::searchAliases($acc->getAccountname(), $this->request->getParameter('search'));
    else
      $aliases = Aliases::callFinder('findByAccountname', array('accountname' => $acc->getAccountname()), 'domainname ASC, aliasname ASC');

    $renderEngine = new myMVC_SmartyEngine(dirname(__FILE__).'/../pages/templates',
                                           dirname(__FILE__).'/../pages/templates_c');
    $view = new SearchView($renderEngine, $this->session, 'searchAliases.tpl');
    $view->setObject('aliases', $aliases);
    $controller->setCurrentLanguage();
    if($view->preRender())
      $html = $view->fetch();
    else
    {
      $msg = myMVC_Logger::doLog(__METHOD__, __LINE__, 'SearchView::preRender() returned false. Unable to display page');
      throw new myMVC_MVCException($msg);
    }

    print json_encode(array('result' => 'OK', 'html' => $html));
    exit;
  }

  /**
   * Called if user data are not valid (check with validate)
   *
   * @return myMVC_HttpResponse     Response code for the next Action or View
   */
  public function manageInvalidDataAjax()
  {
    header('Content-Type: application/json');
    $errors = $this->session->getFromNamespace('FORMCHECK', 'ERRORS');
    $ret = array('result' => 'ERROR', 'error' => $errors['search']);
    print json_encode($ret);
    exit;
  }
}
?>
